@@ -2,8 +2,8 @@
 #
 # ibus - The Input Bus
 #
-# Copyright (c) 2007-2010 Peng Huang <shawn.p.huang@gmail.com>
-# Copyright (c) 2007-2010 Red Hat, Inc.
+# Copyright (c) 2007-2014 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright (c) 2007-2014 Red Hat, Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -29,12 +29,14 @@ from gi.repository import Pango
 from i18n import _, N_
 
 class EngineAbout(Gtk.Dialog):
-    def __init__(self, enginedesc):
-        self.__engine_desc = enginedesc
-        super(EngineAbout, self).__init__(_("About"), None,
-                Gtk.DialogFlags.MODAL,
-                (Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE))
+    def __init__(self, engine, transient_for = None):
+        self.__engine_desc = engine
+        super(EngineAbout, self).__init__(
+                title = _("About"),
+                transient_for = transient_for)
 
+        buttons = (_("_Close"), Gtk.ResponseType.CLOSE)
+        self.add_buttons(*buttons)
         self.__init_ui()
 
     def __init_ui(self):
@@ -99,14 +101,19 @@ class EngineAbout(Gtk.Dialog):
         text_buffer.create_tag("left_margin_32",
                         left_margin=32)
 
-    def __load_icon(self, icon):
+    def __load_icon(self, icon_name):
         try:
-            pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(icon, 48, 48, True)
+            pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(icon_name,
+                                                             48,
+                                                             48,
+                                                             True)
         except:
             theme = Gtk.IconTheme.get_default()
-            icon = theme.lookup_icon("ibus-engine", 48, 0)
+            icon = theme.lookup_icon(icon_name, 48, 0)
             if icon == None:
-                icon = theme.lookup_icon(Gtk.STOCK_MISSING_IMAGE, 48, 0)
+                icon = theme.lookup_icon("ibus-engine", 48, 0)
+            if icon == None:
+                icon = theme.lookup_icon("image-missing", 48, 0)
             pixbuf = icon.load_icon()
         return pixbuf
 
