@@ -2,8 +2,8 @@
 #
 # ibus - The Input Bus
 #
-# Copyright (c) 2007-2010 Peng Huang <shawn.p.huang@gmail.com>
-# Copyright (c) 2007-2010 Red Hat, Inc.
+# Copyright (c) 2007-2014 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright (c) 2007-2014 Red Hat, Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -20,13 +20,21 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 # USA
 
+# for python2
+from __future__ import print_function
+
 import os
 import signal
 import sys
 import time
 
-from gi.repository import Gio
 from gi.repository import GLib
+# set_prgname before importing other modules to show the name in warning
+# messages when import modules are failed.
+GLib.set_prgname('ibus-setup')
+
+from gi.repository import GdkX11
+from gi.repository import Gio
 from gi.repository import Gtk
 from gi.repository import IBus
 from os import path
@@ -34,6 +42,7 @@ from os import path
 import keyboardshortcut
 import locale
 from enginecombobox import EngineComboBox
+from enginedialog import EngineDialog
 from enginetreeview import EngineTreeView
 from engineabout import EngineAbout
 from i18n import DOMAINNAME, _, N_, init as i18n_init
@@ -45,7 +54,7 @@ from i18n import DOMAINNAME, _, N_, init as i18n_init
     COLUMN_VISIBLE,
     COLUMN_ICON,
     COLUMN_DATA,
-) = range(6)
+) = list(range(6))
 
 (
     DATA_NAME,
@@ -57,7 +66,7 @@ from i18n import DOMAINNAME, _, N_, init as i18n_init
     DATA_EXEC,
     DATA_STARTED,
     DATA_PRELOAD
-) = range(9)
+) = list(range(9))
 
 class Setup(object):
     def __flush_gtk_events(self):
@@ -67,10 +76,12 @@ class Setup(object):
     def __init__(self):
         super(Setup, self).__init__()
 
-        self.__settings_general = Gio.Settings("org.freedesktop.ibus.general");
+        self.__settings_general = Gio.Settings(
+                schema = "org.freedesktop.ibus.general");
         self.__settings_hotkey = Gio.Settings(
-                "org.freedesktop.ibus.general.hotkey");
-        self.__settings_panel = Gio.Settings("org.freedesktop.ibus.panel");
+                schema = "org.freedesktop.ibus.general.hotkey");
+        self.__settings_panel = Gio.Settings(
+                schema = "org.freedesktop.ibus.panel");
 
         # IBus.Bus() calls ibus_bus_new().
         # Gtk.Builder().add_from_file() also calls ibus_bus_new_async()
@@ -81,6 +92,9 @@ class Setup(object):
         self.__bus = None
         self.__init_bus()
 
+        # Gtk.ListBox has been available since gtk 3.10
+        self.__has_list_box = hasattr(Gtk, 'ListBox')
+
         gtk_builder_file = path.join(path.dirname(__file__), "./setup.ui")
         self.__builder = Gtk.Builder()
         self.__builder.set_translation_domain(DOMAINNAME)
@@ -190,7 +204,11 @@ class Setup(object):
         # init engine page
         self.__engines = self.__bus.list_engines()
         self.__combobox = self.__builder.get_object("combobox_engines")
-        self.__combobox.set_engines(self.__engines)
+        if self.__has_list_box:
+            self.__combobox.set_no_show_all(True)
+            self.__combobox.hide()
+        else:
+            self.__combobox.set_engines(self.__engines)
 
         tmp_dict = {}
         for e in self.__engines:
@@ -202,7 +220,11 @@ class Setup(object):
         self.__treeview.set_engines(engines)
 
         button = self.__builder.get_object("button_engine_add")
-        button.connect("clicked", self.__button_engine_add_cb)
+        if self.__has_list_box:
+            button.set_sensitive(True)
+            button.connect("clicked", self.__button_engine_add_cb)
+        else:
+            button.connect("clicked", self.__button_engine_add_cb_deprecate)
         button = self.__builder.get_object("button_engine_remove")
         button.connect("clicked", lambda *args:self.__treeview.remove_engine())
         button = self.__builder.get_object("button_engine_up")
@@ -228,6 +250,7 @@ class Setup(object):
         # add icon search path
         self.__window = self.__builder.get_object("window_preferences")
         self.__window.connect("delete-event", Gtk.main_quit)
+        self.__window.connect("notify::window", self.__gdk_window_set_cb)
 
         self.__button_close = self.__builder.get_object("button_close")
         self.__button_close.connect("clicked", Gtk.main_quit)
@@ -243,6 +266,10 @@ class Setup(object):
         self.__init_panel()
         self.__init_general()
 
+    def __gdk_window_set_cb(self, object, pspec):
+        str = '%u' % GdkX11.X11Window.get_xid(object.get_window())
+        GLib.setenv('IBUS_SETUP_XID', str, True)
+
     def __combobox_notify_active_engine_cb(self, combobox, property):
         engine = self.__combobox.get_active_engine()
         button = self.__builder.get_object("button_engine_add")
@@ -271,7 +298,7 @@ class Setup(object):
         if prop.name not in ("active-engine", "engines"):
             return
 
-        engines = self.__treeview.get_engines()
+        engines = self.__treeview.get_sorted_engines()
         engine = self.__treeview.get_active_engine()
 
         self.__builder.get_object("button_engine_remove").set_sensitive(engine != None)
@@ -286,17 +313,32 @@ class Setup(object):
             obj.set_sensitive(False)
 
         if prop.name == "engines":
-            engine_names = map(lambda e: e.get_name(), engines)
+            engines = self.__treeview.get_engines()
+            engine_names = [e.get_name() for e in engines]
             self.__settings_general.set_strv('preload-engines', engine_names)
 
     def __button_engine_add_cb(self, button):
+        dialog = EngineDialog(transient_for = self.__window)
+        dialog.set_engines(self.__engines)
+        id = dialog.run()
+
+        if id != Gtk.ResponseType.APPLY:
+            dialog.destroy()
+            return
+
+        engine = dialog.get_selected_engine()
+        dialog.destroy()
+
+        self.__treeview.append_engine(engine)
+
+    def __button_engine_add_cb_deprecate(self, button):
         engine = self.__combobox.get_active_engine()
         self.__treeview.append_engine(engine)
 
     def __button_engine_about_cb(self, button):
         engine = self.__treeview.get_active_engine()
         if engine:
-            about = EngineAbout(engine)
+            about = EngineAbout(engine = engine, transient_for = self.__window)
             about.run()
             about.destroy()
 
@@ -306,7 +348,7 @@ class Setup(object):
         if len(args) == 0:
             return
         name = engine.get_name()
-        if name in self.__engine_setup_exec_list.keys():
+        if name in list(self.__engine_setup_exec_list.keys()):
             try:
                 wpid, sts = os.waitpid(self.__engine_setup_exec_list[name],
                                        os.WNOHANG)
@@ -324,7 +366,7 @@ class Setup(object):
             return
 
         message = _("The IBus daemon is not running. Do you wish to start it?")
-        dlg = Gtk.MessageDialog(type = Gtk.MessageType.QUESTION,
+        dlg = Gtk.MessageDialog(message_type = Gtk.MessageType.QUESTION,
                                 buttons = Gtk.ButtonsType.YES_NO,
                                 text = message)
         id = dlg.run()
@@ -350,7 +392,7 @@ class Setup(object):
                 "  export XMODIFIERS=@im=ibus\n"
                 "  export QT_IM_MODULE=ibus"
                 )
-            dlg = Gtk.MessageDialog(type = Gtk.MessageType.INFO,
+            dlg = Gtk.MessageDialog(message_type = Gtk.MessageType.INFO,
                                     buttons = Gtk.ButtonsType.OK,
                                     text = message)
             id = dlg.run()
@@ -359,7 +401,7 @@ class Setup(object):
         else:
             # Translators: %d == 5 currently
             message = _("IBus daemon could not be started in %d seconds")
-            dlg = Gtk.MessageDialog(type = Gtk.MessageType.INFO,
+            dlg = Gtk.MessageDialog(message_type = Gtk.MessageType.INFO,
                                     buttons = Gtk.ButtonsType.OK,
                                     text = message % timeout)
             id = dlg.run()
@@ -368,11 +410,15 @@ class Setup(object):
             sys.exit(0)
 
     def __shortcut_button_clicked_cb(self, button, name, section, _name, entry):
-        buttons = (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
-                Gtk.STOCK_OK, Gtk.ResponseType.OK)
-        title = _("Select keyboard shortcut for %s") % \
-                _("switching input methods")
-        dialog = keyboardshortcut.KeyboardShortcutSelectionDialog(buttons = buttons, title = title)
+        buttons = (_("_Cancel"), Gtk.ResponseType.CANCEL,
+                   _("_OK"), Gtk.ResponseType.OK)
+        title1 = _("Select keyboard shortcut for %s")
+        # Translators: Title of the window
+        title2 = _("switching input methods")
+        title = title1 % title2
+        dialog = keyboardshortcut.KeyboardShortcutSelectionDialog(
+                title = title, transient_for = self.__window)
+        dialog.add_buttons(*buttons)
         text = entry.get_text()
         if text:
             shortcuts = text.split("; ")
@@ -402,10 +448,11 @@ class Setup(object):
         if data[DATA_STARTED] == False:
             try:
                 self.__bus.register_start_engine(data[DATA_LANG], data[DATA_NAME])
-            except Exception, e:
-                dlg = Gtk.MessageDialog(type = Gtk.MessageType.ERROR,
+            except Exception as e:
+                dlg = Gtk.MessageDialog(message_type = Gtk.MessageType.ERROR,
+                        transient_for = self.__window,
                         buttons = Gtk.ButtonsType.CLOSE,
-                        message_format = str(e))
+                        text = str(e))
                 dlg.run()
                 dlg.destroy()
                 self.__flush_gtk_events()
@@ -413,10 +460,11 @@ class Setup(object):
         else:
             try:
                 self.__bus.register_stop_engine(data[DATA_LANG], data[DATA_NAME])
-            except Exception, e:
-                dlg = Gtk.MessageDialog(type = Gtk.MessageType.ERROR,
+            except Exception as e:
+                dlg = Gtk.MessageDialog(message_type = Gtk.MessageType.ERROR,
+                        transient_for = self.__window,
                         buttons = Gtk.ButtonsType.CLOSE,
-                        message_format = str(e))
+                        text = str(e))
                 dlg.run()
                 dlg.destroy()
                 self.__flush_gtk_events()
@@ -492,7 +540,7 @@ if __name__ == "__main__":
     try:
         locale.setlocale(locale.LC_ALL, '')
     except locale.Error:
-        print >> sys.stderr, "Using the fallback 'C' locale"
+        print("Using the fallback 'C' locale", file=sys.stderr)
         locale.setlocale(locale.LC_ALL, 'C')
 
     i18n_init()
