@@ -1,7 +1,7 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* ibus - The Input Bus
- * Copyright (C) 2013 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2013 Takao Fujiwara <takao.fujiwara1@gmail.com>
+ * Copyright (C) 2013-2014 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2013-2014 Takao Fujiwara <takao.fujiwara1@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,6 +19,17 @@
  * USA
  */
 
+/* This tool converts COMPOSE_FILES[] to ibuscompose.h .
+ * To update ibuscompose.h:
+ * # rm ibuscompose.h
+ * # make ibuscompose.h
+ *
+ * If you copy libX11/nls/??/Compose.pre in xorg git HEAD to
+ * /usr/share/X11/locale/??/Compose , need to convert:
+ * # sed -e 's/^XCOMM/#/' -e 's|X11_LOCALEDATADIR|/usr/share/X11/locale|'
+ *   Compose.pre > /usr/share/X11/locale/foo/Compose
+ */
+
 #include <glib.h>
 #include <glib/gprintf.h>
 #include <locale.h>
@@ -36,8 +47,8 @@
 "/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */\n"          \
 "/* vim:set et sts=4: */\n"                                                   \
 "/* ibus - The Input Bus\n"                                                   \
-" * Copyright (C) 2013 Peng Huang <shawn.p.huang@gmail.com>\n"                \
-" * Copyright (C) 2013 Takao Fujiwara <takao.fujiwara1@gmail.com>\n"          \
+" * Copyright (C) 2013-2014 Peng Huang <shawn.p.huang@gmail.com>\n"           \
+" * Copyright (C) 2013-2014 Takao Fujiwara <takao.fujiwara1@gmail.com>\n"     \
 " *\n"                                                                        \
 " * This library is free software; you can redistribute it and/or\n"          \
 " * modify it under the terms of the GNU Lesser General Public\n"             \
@@ -647,7 +658,7 @@ check_duplicated_compose (GList *compose_table)
         int i;
         int n_compose = 0;
 
-        for (i = 0; i <= GTK_MAX_COMPOSE_LEN + 2; i++) {
+        for (i = 0; i < GTK_MAX_COMPOSE_LEN + 2; i++) {
             keysyms[i] = 0;
         }
 
