@@ -21,6 +21,7 @@
  */
 #include <stdlib.h>
 #include "ibusenginedesc.h"
+#include "ibusinternal.h"
 #include "ibusxml.h"
 
 enum {
@@ -554,29 +555,44 @@ ibus_engine_desc_deserialize (IBusEngineDesc *desc,
      * you should not change the serialized order of name, longname,
      * description, ... because the order is also used in other applications
      * likes ibus-qt. */
-    g_variant_get_child (variant, retval++, "s", &desc->priv->name);
-    g_variant_get_child (variant, retval++, "s", &desc->priv->longname);
-    g_variant_get_child (variant, retval++, "s", &desc->priv->description);
-    g_variant_get_child (variant, retval++, "s", &desc->priv->language);
-    g_variant_get_child (variant, retval++, "s", &desc->priv->license);
-    g_variant_get_child (variant, retval++, "s", &desc->priv->author);
-    g_variant_get_child (variant, retval++, "s", &desc->priv->icon);
-    g_variant_get_child (variant, retval++, "s", &desc->priv->layout);
+    ibus_g_variant_get_child_string (variant, retval++,
+                                     &desc->priv->name);
+    ibus_g_variant_get_child_string (variant, retval++,
+                                     &desc->priv->longname);
+    ibus_g_variant_get_child_string (variant, retval++,
+                                     &desc->priv->description);
+    ibus_g_variant_get_child_string (variant, retval++,
+                                     &desc->priv->language);
+    ibus_g_variant_get_child_string (variant, retval++,
+                                     &desc->priv->license);
+    ibus_g_variant_get_child_string (variant, retval++,
+                                     &desc->priv->author);
+    ibus_g_variant_get_child_string (variant, retval++,
+                                     &desc->priv->icon);
+    ibus_g_variant_get_child_string (variant, retval++,
+                                     &desc->priv->layout);
     g_variant_get_child (variant, retval++, "u", &desc->priv->rank);
     /* The serialized order should be kept. */
-    g_variant_get_child (variant, retval++, "s", &desc->priv->hotkeys);
-    g_variant_get_child (variant, retval++, "s", &desc->priv->symbol);
-    g_variant_get_child (variant, retval++, "s", &desc->priv->setup);
+    ibus_g_variant_get_child_string (variant, retval++,
+                                     &desc->priv->hotkeys);
+    ibus_g_variant_get_child_string (variant, retval++,
+                                     &desc->priv->symbol);
+    ibus_g_variant_get_child_string (variant, retval++,
+                                     &desc->priv->setup);
     if (g_variant_n_children (variant) < retval + 2)
         return retval;
-    g_variant_get_child (variant, retval++, "s", &desc->priv->layout_variant);
-    g_variant_get_child (variant, retval++, "s", &desc->priv->layout_option);
+    ibus_g_variant_get_child_string (variant, retval++,
+                                     &desc->priv->layout_variant);
+    ibus_g_variant_get_child_string (variant, retval++,
+                                     &desc->priv->layout_option);
     if (g_variant_n_children (variant) < retval + 1)
         return retval;
-    g_variant_get_child (variant, retval++, "s", &desc->priv->version);
+    ibus_g_variant_get_child_string (variant, retval++,
+                                     &desc->priv->version);
     if (g_variant_n_children (variant) < retval + 1)
         return retval;
-    g_variant_get_child (variant, retval++, "s", &desc->priv->textdomain);
+    ibus_g_variant_get_child_string (variant, retval++,
+                                     &desc->priv->textdomain);
 
     return retval;
 }
